/*
 * DirectoryException.java
 *
 * This source file is part of the FoundationDB open source project
 *
 * Copyright 2013-2018 Apple Inc. and the FoundationDB project authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.apple.foundationdb.directory;

import java.util.List;

/**
 * An {@link Exception} that is thrown by the directory layer corresponding
 * to a specific path.
 */
@SuppressWarnings("serial")
public class DirectoryException extends RuntimeException {
	/**
	 * The directory path that was responsible for the error.
	 */
	public final List<String> path;

	DirectoryException(String baseMsg, List<String> path) {
		super(baseMsg + ": path=" + DirectoryUtil.pathStr(path));
		this.path = path;
	}
}
